(function() {
  var CustomFile, File, Promise, Resource, createFileObject, defineProperty, extend, fileNameSensitive, getPrototypeOf, inherits, isArray, isFunction, isObject, isString, loadCfgFile, loadCfgFolder, markdownExts, matter, setImmediate, setPrototypeOf,
    __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

  fileNameSensitive = require('fs-file-name-sensitive');

  CustomFile = require('custom-file');

  File = require('custom-file/lib/advance');

  inherits = require('inherits-ex/lib/inherits');

  getPrototypeOf = require('inherits-ex/lib/getPrototypeOf');

  setPrototypeOf = require('inherits-ex/lib/setPrototypeOf');

  matter = require('front-matter-markdown/lib/');

  loadCfgFile = require('load-config-file');

  loadCfgFolder = require('load-config-folder');

  extend = require('util-ex/lib/_extend');

  isObject = require('util-ex/lib/is/type/object');

  isString = require('util-ex/lib/is/type/string');

  isArray = require('util-ex/lib/is/type/array');

  isFunction = require('util-ex/lib/is/type/function');

  defineProperty = require('util-ex/lib/defineProperty');

  Promise = require('bluebird');

  createFileObject = require('./create-file-object');

  setImmediate = setImmediate || process.nextTick;

  Promise.promisifyAll(File, {
    filter: function(name, fn) {
      return name === 'load';
    }
  });

  markdownExts = ['.txt', '.md', '.mdown', '.markdown', '.mkd', '.mkdn', '.mdwn', '.mdtext', '.mdtxt', '.text'];

  module.exports = Resource = (function() {
    var fs, path;

    inherits(Resource, File);

    Resource.setFileSystem = function(value) {
      CustomFile.setFileSystem(value);
      Resource.prototype._updateFS();
      return Resource;
    };

    fs = null;

    path = null;

    function Resource(aPath, aOptions, done) {
      if (!(this instanceof Resource)) {
        return new Resource(aPath, aOptions, done);
      }
      Resource.__super__.constructor.apply(this, arguments);
    }

    Resource.defineProperties = File.defineProperties;

    matter.setOptionAlias('toc', ['isDir', 'isDirectory']);

    matter.setOptionAlias('heading', ['dirHeading', 'dirHeadings']);

    matter.setOptionAlias('headingsAsToc', 'headingsAsToc');

    File.defineProperties(Resource, extend({
      isDir: {
        type: 'Boolean',
        alias: ['isDirectory']
      }
    }, File.prototype.$attributes));

    defineProperty(Resource.prototype, 'parent', void 0, {
      get: function() {
        var result;
        result = getPrototypeOf(this);
        if (result === Object.prototype) {
          result = null;
        }
        return result;
      }
    });

    Resource.prototype.createFileObject = function(aOptions, aFilter) {
      var result;
      aOptions.cwd = this.cwd;
      aOptions.base = this.base;
      if (aFilter == null) {
        aFilter = this.filter;
      }
      if (!isFunction(aFilter) || aFilter.call(this, aOptions)) {
        result = createFileObject(this, aOptions);
      }
      return result;
    };

    Resource.prototype._assign = function(aOptions, aExclude) {
      var k, v, vAttrs, _results;
      vAttrs = this.getProperties();
      _results = [];
      for (k in aOptions) {
        v = aOptions[k];
        if (k === 'load' || k === 'read' || k === 'buffer' || k === 'text') {
          continue;
        }
        if ((vAttrs[k] != null) || __indexOf.call(aExclude, k) >= 0) {
          continue;
        }
        if (isObject(v) && isObject(v['<'])) {
          v = v['<'];
          if (isArray(this[k])) {
            if (isArray(v)) {
              v = v.concat(this[k]);
            } else {
              v = this[k].concat(v);
            }
          } else if (isObject(this[k])) {
            v = extend({}, this[k], v);
          }
        }
        _results.push(this[k] = v);
      }
      return _results;
    };

    Resource.prototype._updateFS = function(aFS) {
      Resource.__super__._updateFS.call(this, aFS);
      if (fs !== this.fs || !fs) {
        fs = this.fs;
      }
      if (fs) {
        loadCfgFolder.setFileSystem(fs);
        if (!path) {
          path = fs.path;
        }
      }
    };

    Resource.prototype.inspect = function() {
      var name;
      name = 'File';
      if (this.isDirectory()) {
        name = 'Folder';
      }
      if (!this.loaded()) {
        name += '?';
      }
      return '<' + name + ' ' + this._inspect() + '>';
    };

    Resource.prototype.isDirectory = function() {
      var result;
      if (this.hasOwnProperty('isDir') && this.isDir !== void 0) {
        result = this.isDir;
      } else {
        result = Resource.__super__.isDirectory.call(this);
      }
      return result;
    };

    Resource.prototype.toObject = function(options, aExclude) {
      if (isString(aExclude)) {
        aExclude = [aExclude];
      } else if (!isArray(aExclude)) {
        aExclude = [];
      }
      if (!this.hasOwnProperty('contents')) {
        aExclude.push('contents');
      }
      if (!this.hasOwnProperty('stat')) {
        aExclude.push('stat');
      }
      return this.exportTo(options, aExclude);
    };

    Resource.prototype.getContentSync = function(aOptions) {
      if (!isObject(aOptions)) {
        aOptions = {};
      }
      if (!((aOptions.overwrite != null) || this.loaded())) {
        aOptions.overwrite = true;
      }
      return Resource.__super__.getContentSync.call(this, aOptions);
    };

    Resource.prototype.getContent = function(aOptions, done) {
      if (isFunction(aOptions)) {
        done = aOptions;
      }
      if (!isObject(aOptions)) {
        aOptions = {};
      }
      if (!((aOptions.overwrite != null) || this.loaded())) {
        aOptions.overwrite = true;
      }
      return Resource.__super__.getContent.call(this, aOptions, done);
    };

    Resource.prototype.frontMatter = function(aText, aOptions) {
      var result, _ref;
      if (_ref = this.extname, __indexOf.call(markdownExts, _ref) >= 0) {
        result = matter(aText.toString(), aOptions);
      }
      return result;
    };

    Resource.prototype.convertVirtualFolder = function(aContents) {
      var k, _i, _len;
      for (_i = 0, _len = aContents.length; _i < _len; _i++) {
        k = aContents[_i];
        if (k.path != null) {
          if (this.stat.isDirectory()) {
            k.base = this.path;
          } else {
            k.base = path.dirname(this.path);
          }
          k.path = path.resolve(k.base, k.path);
          setPrototypeOf(k, this);
        }
      }
      return aContents;
    };

    Resource.prototype.loadConfig = function(aOptions, aContents, done) {
      var processCfg, that, vFrontConf, vOptions;
      that = this;
      processCfg = function(err, aConfig) {
        var vDir;
        if (err) {
          return done(err);
        }
        if (aConfig) {
          that.assign(aConfig, 'contents');
          if (vDir = that.isDirectory()) {
            if (aConfig.contents) {
              that.convertVirtualFolder(aConfig.contents);
              aContents = aConfig.contents;
            }
          }
        }
        if (vDir == null) {
          vDir = that.isDirectory();
        }
        if (vDir && aOptions.recursive) {
          aContents.forEach(function(f) {
            if ((f instanceof Resource) && f.isDirectory()) {
              return f.loadSync(aOptions);
            }
          });
        }
        done(err, aConfig);
      };
      if (!aOptions.stat.isDirectory()) {
        vFrontConf = this.frontMatter(aContents, aOptions);
        vOptions = {
          exclude: aOptions.path
        };
        if (aOptions.configurators) {
          vOptions.configurators = aOptions.configurators;
        }
        return loadCfgFile(aOptions.path, vOptions, function(err, result) {
          if (err) {
            return done(err);
          }
          if (vFrontConf && vFrontConf.skipSize) {
            if (!isObject(result)) {
              result = {};
            }
            result = extend(result, vFrontConf);
            if (result.contents) {
              aOptions.skipSize = -vFrontConf.skipSize;
            } else {
              aOptions.skipSize = vFrontConf.skipSize;
            }
          }
          return processCfg(err, result);
        });
      } else {
        return loadCfgFolder(aOptions.path, aOptions, processCfg);
      }
    };

    Resource.prototype.loadConfigSync = function(aOptions, aContents) {
      var result, vFrontConf, vIsDir, vOptions;
      if (!aOptions.stat.isDirectory()) {
        vFrontConf = this.frontMatter(aContents.toString(), aOptions);
        vOptions = {
          exclude: aOptions.path
        };
        if (aOptions.configurators) {
          vOptions.configurators = aOptions.configurators;
        }
        result = loadCfgFile(aOptions.path, vOptions);
        if (!isObject(result)) {
          result = {};
        }
        if (vFrontConf && vFrontConf.skipSize) {
          result = extend(result, vFrontConf);
          if (result.contents) {
            aOptions.skipSize = -vFrontConf.skipSize;
          } else {
            aOptions.skipSize = vFrontConf.skipSize;
          }
        }
      } else {
        result = loadCfgFolder(aOptions.path, aOptions);
      }
      if (result) {
        this.assign(result, 'contents');
        if (result.$cfgPath) {
          this.$cfgPath = result.$cfgPath;
        }
        if (vIsDir = this.isDirectory()) {
          if (result.contents) {
            this.convertVirtualFolder(result.contents);
            aContents = result.contents;
          }
        }
      }
      if (vIsDir == null) {
        vIsDir = this.isDirectory();
      }
      if (vIsDir && aOptions.recursive) {
        aContents.forEach(function(f) {
          if ((f instanceof Resource) && f.isDirectory()) {
            return f.loadSync(aOptions);
          }
        });
      }
      return result;
    };

    Resource.prototype._getBufferSync = function(aFile) {
      var conf, result, vCfgPath, vDir, vIsFileNameInsensitive;
      result = Resource.__super__._getBufferSync.call(this, aFile);
      conf = this.loadConfigSync(aFile, result);
      if (conf) {
        if (conf.contents) {
          result = conf.contents;
        }
        if (conf.$cfgPath) {
          vIsFileNameInsensitive = !fileNameSensitive(this.cwd);
          vCfgPath = conf.$cfgPath;
          if (vIsFileNameInsensitive) {
            vCfgPath = vCfgPath.toLowerCase();
          }
          if (this.isDirectory()) {
            result = result.filter(function(f) {
              var vPath;
              vPath = f.path;
              if (vIsFileNameInsensitive) {
                vPath = vPath.toLowerCase();
              }
              return vPath !== vCfgPath;
            });
          } else if ((vDir = this.parent)) {
            vDir.contents = vDir.contents.filter(function(f) {
              var vPath;
              vPath = f.path;
              if (vIsFileNameInsensitive) {
                vPath = vPath.toLowerCase();
              }
              return vPath !== vCfgPath;
            });
          }
        }
      }
      return result;
    };

    Resource.prototype._getBuffer = function(aFile, done) {
      var that, vIsFileNameInsensitive;
      that = this;
      vIsFileNameInsensitive = !fileNameSensitive(that.cwd);
      return Resource.__super__._getBuffer.call(this, aFile, function(err, result) {
        if (err) {
          return done(err);
        }
        return that.loadConfig(aFile, result, function(err, conf) {
          var vCfgPath, vDir;
          if (err) {
            return done(err);
          }
          if (conf) {
            if (conf.contents) {
              result = conf.contents;
            }
            if (conf.$cfgPath) {
              vCfgPath = conf.$cfgPath;
              if (vIsFileNameInsensitive) {
                vCfgPath = vCfgPath.toLowerCase();
              }
              if (that.isDirectory()) {
                result = result.filter(function(f) {
                  var vPath;
                  vPath = f.path;
                  if (vIsFileNameInsensitive) {
                    vPath = vPath.toLowerCase();
                  }
                  return vPath !== vCfgPath;
                });
              } else if ((vDir = that.parent)) {
                vDir.contents = vDir.contents.filter(function(f) {
                  var vPath;
                  vPath = f.path;
                  if (vIsFileNameInsensitive) {
                    vPath = vPath.toLowerCase();
                  }
                  return vPath !== vCfgPath;
                });
              }
            }
          }
          return done(err, result);
        });
      });
    };

    return Resource;

  })();

}).call(this);

//# sourceMappingURL=index.js.map
